import datetime

from botocore.signers import CloudFrontSigner
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.primitives.asymmetric import padding

from cloud_resources import credentials as creds
from cloud_resources.error_handler import SigningError


def rsa_signer(message):
    """
    Helps to sign the required information with RSA

    Helps in making the part of the url which contains the signed information.
    The message parameter conatins information with which we will sign the url.
    The usual information used to sign the url are:
        1) the object to be signed. Ex: The storage directory name or the S3 key
        2) Expiry date of the url
        3) IP address from which the data is originating

    This function is passed to the signing function in the cloudfront
    so cloudfront api will be passing the argument message to the function

    Parameters
    ----------
    message : Dict
        The format accepted for this dict is:
        {
            "Statement": [{
                "Resource":"URL or stream name of the object",
                "Condition":{
                    "DateLessThan":{"AWS:EpochTime":required ending
                                    date and time in Unix time format and UTC},
                    "DateGreaterThan":{"AWS:EpochTime":optional beginning date
                                     and time in Unix time format and UTC},
                    "IpAddress":{"AWS:SourceIp":"optional IP address"}
                }
            }]
        }

    Returns
    -------
    String
        Signed information

    """
    if creds.SIGNING_PEM_FILE_PATH is None:
        raise SigningError("Signing File Path not specified")
    with open(creds.SIGNING_PEM_FILE_PATH, 'rb') as key_file:
        private_key = serialization.load_pem_private_key(
            key_file.read(),
            password=None,
            backend=default_backend())
    signer = private_key.signer(padding.PKCS1v15(), hashes.SHA1())
    signer.update(message)
    return signer.finalize()


def sign_url(url, key_id):
    """
    Signes information and attaches it to the given url

    Signes the information that is provided with the private key
    information that is used are
    1) the object that need to be signed eg. the S3 key
    2) expiry date of the url
    After signing the information with the private key it will be attached
    to the url.

    Parameters
    ----------
    url : String
        URL for which signed url should be created
    key_id: String
        Key ID for signing the URL

    Returns
    -------
    String
        Signed URL
    """
    if url is None or key_id is None:
        raise SigningError("No URL or KeyID specified")
    expire_time = datetime.datetime.now() + datetime.timedelta(hours=1)
    cloudfront_signer = CloudFrontSigner(key_id, rsa_signer)

    # Create a signed url that will be valid until the specfic expiry date
    # provided using a custom policy.
    # if more option to signing has to be added add in the below function
    # check
    # https://github.com/boto/botocore/blob/8483d71daf66234644e760050ab2371eae95eac9/botocore/signers.py#L348
    # for more information on how to add it
    policy = cloudfront_signer.build_policy(url, date_less_than=expire_time)
    signed_url = cloudfront_signer.generate_presigned_url(url, policy=policy)
    return signed_url
