from __future__ import division
from builtins import range
from past.utils import old_div
from boto3.dynamodb.conditions import Key
import boto3

from cloud_resources import credentials as cred



def get_client():
    """
    Returns
    -------
    NoSQL DB Client (DynamoDB)
    """
    return boto3.Session(aws_access_key_id=cred.DYNAMO_ACCESS_KEY,
                         aws_secret_access_key=cred.DYNAMO_SECRET_KEY,
                         region_name=cred.DYNAMO_REGION).resource('dynamodb')


def write_data_batch(Items, table_name, cut_len=25):
    """
    Write to the NoSQL (DynamoDB) DB in the batch mode

    Parameters
    ----------
    Items: Dict
        Data to be stored
    table_name: String
        Name of the table
    cut_len: Integer
        Batch size
    """
    client = get_client()
    table = client.Table(table_name)

    for i in range((old_div(len(Items), cut_len)) + 1):
        batch_item = Items[cut_len * i:cut_len*i + cut_len]
        with table.batch_writer() as batch:
            for Item in batch_item:
                batch.put_item(Item=Item)


def get_data(table_name, key_name, query_filter):
    """
    Returns data from the NoSQL DB given a table_name and key_name along with
    a filter param

    table_name: String
        Name of the table (Object) in NoSQL DB
    key_name: String
        Name of the key in DB
    query_filter: String
        Value to filter against

    Returns
    -------
    Object
        Data stored in the DB
    """
    client = get_client()
    table = client.Table(table_name)
    filtering_exp = Key(key_name).eq(query_filter)
    response = table.query(KeyConditionExpression=filtering_exp)
    return response['Items']


def get_all_indexes(table_name, getter_name):
    """
    Returns the data for a complete scan for a given key in a given table

    Parameters
    ----------
    table_name: String
        Name of the table (Object) in NoSQL DB
    key_name: String
        Name of the key in DB

    Returns
    -------
    Object
        All data stored in the DB for given key
    """
    client = get_client()
    table = client.Table(table_name)
    return table.scan().get(getter_name)


def get_all_indexes_conditional(table_name, getter_name, query_filter):
    """
    Returns the data for a complete scan for a given key in a given table

    Parameters
    ----------
    table_name: String
        Name of the table (Object) in NoSQL DB
    key_name: String
        Name of the key in DB

    Returns
    -------
    Object
        All data stored in the DB for given key
    """
    client = get_client()
    table = client.Table(table_name)
    return table.scan(FilterExpression=boto3.dynamodb.conditions.
                      Attr(getter_name).eq(query_filter)).get('Items')
