from builtins import str
import json

import boto3
from botocore.exceptions import ClientError, ParamValidationError

from cloud_resources.error_handler import SQSError


def add_message(queue_name, message, region_name):
    """
    Writes the message to the queue

    queue_name: String
        Name of the queue to write message on.
    message: String
        Message to write to the queue
    region_name: String
        Region to find the queue in.
    """
    try:
        client = boto3.client('sqs', region_name=region_name)
        queue_url = client.get_queue_url(QueueName=queue_name).get('QueueUrl')
        client.send_message(
            QueueUrl=queue_url,
            MessageBody=json.dumps(message)
        )

    except ClientError as e:
        message = "Unable to add message to the queue: "
        raise SQSError(message + str(e))


def create_queue(queue_name, region_name):
    """
    Used to create the queue.

    Parameters
    -----------
    queue_name: String
        name of the queue which is to be created

    Returns
    -----------
    queue_url: String
        URL of the queue created
    """
    try:
        client = boto3.client('sqs', region_name=region_name)
        queue_url = client.create_queue(QueueName=queue_name)
    except (ClientError, KeyError, ParamValidationError) as e:
        raise SQSError("Unable  to create queue: " + str(e))
    return queue_url


def delete_message(queue_name, message_receipt_handle, region_name):
    """
    Delete a particular message from the queue.

    Parameters
    ----------
    queue_name: String
        Name of the queue
    message_receipt_handle: String
        ReceiptHandle of the message in queue to be deleted
    region_name: String
        Name of the region where queue exists
    """
    try:
        client = boto3.client('sqs', region_name=region_name)
        queue_url = client.get_queue_url(QueueName=queue_name).get('QueueUrl')
        client.delete_message(
            QueueUrl=queue_url,
            ReceiptHandle=message_receipt_handle
        )
    except ClientError as e:
        raise SQSError("Unable to delete the message: " + e.message)


def retrieve_messages(queue_name, region_name, num_messages,
                      visibility_timeout, wait_time_seconds):
    """
    Returns message from the queue given its name.

    The function fetches a given number of messages from the queue, given a
    visibility_timeout and wait_time in seconds.

    Parameters
    ----------
    queue_name: String
        Name of the queue
    region_name: String
        Region where queue exists
    num_messages: Integer
        Number of messages to fetch from the queue
    visibility_timeout: Integer
        Visibility Timeout for the messages
    wait_time_seconds: Integer
        Time to wait for the messages in seconds

    Returns
    -------
    Dict: messages
        Dictionary containing the messages with required attributes.
    """
    try:
        client = boto3.client('sqs', region_name=region_name)
        queue_url = client.get_queue_url(QueueName=queue_name).get('QueueUrl')
        messages = client.receive_message(
            QueueUrl=queue_url,
            MaxNumberOfMessages=num_messages,
            VisibilityTimeout=visibility_timeout,
            WaitTimeSeconds=wait_time_seconds
        )
        return messages
    except ClientError as e:
        raise SQSError("Unable to retrieve messages " + e.message)
